# 25 "cil/src/logic/logic_preprocess.mll"
 
  open Lexing
  type end_of_buffer = NEWLINE | SPACE | CHAR
  let preprocess_buffer = Buffer.create 1024
  let output_buffer = Buffer.create 1024
  let beg_of_line = Buffer.create 8
  let blacklisted_macros = [ "__STDC__"; "__STDC_HOSTED__"; "assert"]
  let is_newline = ref CHAR
  let curr_file = ref ""
  let curr_line = ref 1
  let has_annot = ref false

  let reset () =
    Buffer.clear preprocess_buffer;
    Buffer.clear output_buffer;
    Buffer.clear beg_of_line;
    is_newline := CHAR;
    curr_file := "";
    curr_line := 1;
    has_annot := false

  let backslash = "__ANNOT_BACKSLASH__"
  let annot_content = "__ANNOT_CONTENT__"

  let re_backslash = Str.regexp_string backslash
  let re_annot_content = Str.regexp_string annot_content

  (* Delimiters for the various annotations in the preprocessing buffer.
     We have one delimiter for the beginning of an annotation (to discard
     #defines along the way), and three delimiters for the various ways
     an annotation can end:
      - on a normal line
      - with a newline
      - with a newline inside a comment (only for one-line annotations)
     When preprocessed annotations are inserted back in the main file, this will
     result in distinct translation to preserve line numbers while avoiding
     ill-formed annotations.
  *)
  let annot_beg =         "////////////////__ANNOT_BEG__"
  let annot_end =         "////////////////__ANNOT_END__"
  let annot_end_nl  =     "////////////////__ANNOT_END_NL__"
  let annot_end_comment = "////////////////__ANNOT_END_COMMENT__"

  let abort_preprocess reason =
    let source = {Lexing.dummy_pos with Lexing.pos_fname = !curr_file;
                  pos_lnum = !curr_line;}
    in
    Kernel.error ~source
      "Can't preprocess annotation: %s\nSome annotations will be kept as is"
      reason

  let next_preprocessed file =
    let content = Buffer.create 80 in
    let rec ignore_content () =
      let s = input_line file in
      if s <> annot_beg then ignore_content ()
    in
    let rec get_annot first =
      let s = input_line file in
      if s = annot_end then false, Buffer.contents content
      else if s = annot_end_nl then true, Buffer.contents content
      else if s = annot_end_comment then begin
        Buffer.add_char content '\n';
        false, Buffer.contents content
      end else begin
        if not first then Buffer.add_char content '\n';
        Buffer.add_string content s;
        get_annot false
      end
    in
    let replace_backslash s = Str.global_replace re_backslash "\\\\" s in
    try
      ignore_content ();
      ignore (input_line file); (* ignore the #line directive *)
      let with_nl, content = get_annot true in
      with_nl, replace_backslash content
    with End_of_file ->
      Kernel.fatal
        "too few annotations in result file while pre-processing annotations"

  let output_result outfile preprocessed content =
    let rec aux = function
      | [] -> ()
      | [s] -> output_string outfile s
      | content :: rem ->
          output_string outfile content;
          output_string outfile "/*@";
          let with_nl, pp_content = next_preprocessed preprocessed in
          output_string outfile pp_content;
          output_string outfile "*/";
          if with_nl then output_char outfile '\n';
          aux rem
    in aux content

  let preprocess_annots suffix cpp outfile =
    if !has_annot then begin
      let debug =
        Kernel.debug_atleast 3 ||
          Kernel.Debug_category.exists (fun x -> x = "parser")
      in
      let ppname =
        try Extlib.temp_file_cleanup_at_exit ~debug "ppannot" suffix
        with Extlib.Temp_file_error s ->
          Kernel.abort
            "Could not open temporary file for logic pre-processing: %s" s
      in
      let ppfile = open_out ppname in
      Buffer.output_buffer ppfile preprocess_buffer;
      close_out ppfile;
      let cppname = Extlib.temp_file_cleanup_at_exit ~debug "cppannot" suffix in
      let res = Sys.command (cpp ppname cppname) in
      let result_file =
        if res <> 0 then begin
          abort_preprocess "Preprocessor call exited with an error";
          if not debug then Extlib.safe_remove cppname;
          ppname
        end else cppname
      in
      let result = open_in result_file in
      let content =
        Str.split_delim re_annot_content (Buffer.contents output_buffer)
      in
      output_result outfile result content;
    end else begin
      Buffer.output_buffer outfile output_buffer
    end;
    flush outfile

  let add_preprocess_line_info () =
    Printf.bprintf
      preprocess_buffer "# %d %s \n%s   "
      !curr_line !curr_file (Buffer.contents beg_of_line);
    Buffer.clear beg_of_line

  let make_newline () =
    incr curr_line;
    Buffer.clear beg_of_line

# 141 "cil/src/logic/logic_preprocess.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\245\255\246\255\247\255\248\255\249\255\001\000\004\000\
    \017\000\000\000\031\000\000\000\000\000\000\000\001\000\001\000\
    \002\000\081\000\160\000\000\000\006\000\006\000\004\000\008\000\
    \018\001\028\000\010\000\030\000\031\000\254\255\029\000\032\000\
    \252\255\000\000\253\255\250\255\001\000\251\255\081\000\252\255\
    \253\255\003\000\001\000\255\255\254\255\036\000\253\255\003\000\
    \255\255\254\255\082\000\251\255\253\255\254\255\003\000\252\255\
    \255\255\212\000\251\255\253\255\254\255\050\000\252\255\255\255\
    \044\001\247\255\248\255\249\255\250\255\251\255\252\255\028\000\
    \254\255\029\000\255\255\253\255\130\000\252\255\253\255\030\000\
    \255\255\254\255\213\000\250\255\251\255\083\000\254\255\255\255\
    \252\255\253\255\023\001\251\255\252\255\004\000\254\255\255\255\
    \253\255\243\000\252\255\253\255\254\255\255\255\045\001\250\255\
    \047\000\252\255\253\255\254\255\255\255\251\255\087\000\254\255\
    \255\255\088\000\254\255\255\255";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\010\000\010\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\003\000\255\255\255\255\005\000\255\255\255\255\255\255\
    \255\255\003\000\003\000\255\255\255\255\255\255\255\255\002\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\004\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\004\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\008\000\
    \255\255\008\000\255\255\255\255\255\255\255\255\255\255\003\000\
    \255\255\255\255\255\255\255\255\255\255\005\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\004\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \005\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255";
  Lexing.lex_default = 
   "\001\000\000\000\000\000\000\000\000\000\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\027\000\255\255\027\000\028\000\000\000\035\000\032\000\
    \000\000\255\255\000\000\000\000\255\255\000\000\039\000\000\000\
    \000\000\255\255\255\255\000\000\000\000\046\000\000\000\255\255\
    \000\000\000\000\051\000\000\000\000\000\000\000\255\255\000\000\
    \000\000\058\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \065\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\255\255\000\000\000\000\077\000\000\000\000\000\255\255\
    \000\000\000\000\083\000\000\000\000\000\255\255\000\000\000\000\
    \000\000\000\000\091\000\000\000\000\000\255\255\000\000\000\000\
    \000\000\098\000\000\000\000\000\000\000\000\000\103\000\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\111\000\000\000\
    \000\000\114\000\000\000\000\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\005\000\043\000\000\000\010\000\000\000\000\000\
    \000\000\000\000\000\000\026\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\026\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\003\000\007\000\010\000\056\000\096\000\002\000\
    \010\000\029\000\026\000\031\000\025\000\044\000\048\000\006\000\
    \030\000\026\000\049\000\025\000\008\000\008\000\008\000\008\000\
    \008\000\008\000\008\000\008\000\008\000\008\000\255\255\010\000\
    \028\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\
    \008\000\008\000\008\000\075\000\074\000\081\000\047\000\008\000\
    \008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\
    \008\000\063\000\017\000\040\000\052\000\036\000\109\000\055\000\
    \033\000\112\000\115\000\000\000\020\000\013\000\014\000\017\000\
    \012\000\022\000\015\000\021\000\017\000\024\000\019\000\016\000\
    \009\000\017\000\023\000\000\000\053\000\000\000\000\000\000\000\
    \000\000\011\000\089\000\034\000\037\000\034\000\037\000\000\000\
    \041\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\009\000\080\000\000\000\062\000\000\000\
    \000\000\000\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\079\000\042\000\054\000\088\000\
    \018\000\000\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\000\000\000\000\000\000\000\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\000\000\000\000\000\000\000\000\059\000\087\000\
    \000\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\060\000\086\000\100\000\000\000\018\000\
    \004\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\024\000\255\255\255\255\255\255\255\255\
    \255\255\095\000\101\000\000\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \061\000\085\000\024\000\000\000\000\000\000\000\072\000\108\000\
    \000\000\094\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\
    \008\000\008\000\008\000\008\000\069\000\000\000\066\000\105\000\
    \000\000\255\255\255\255\067\000\106\000\000\000\073\000\112\000\
    \115\000\000\000\000\000\071\000\104\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\070\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\093\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\078\000\000\000\000\000\000\000\000\000\000\000\
    \068\000\107\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\084\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\099\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\092\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\108\000";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\042\000\255\255\007\000\255\255\255\255\
    \255\255\255\255\255\255\026\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\008\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\007\000\054\000\093\000\000\000\
    \010\000\028\000\026\000\006\000\026\000\041\000\045\000\000\000\
    \006\000\008\000\047\000\008\000\007\000\007\000\007\000\007\000\
    \007\000\007\000\007\000\007\000\007\000\007\000\025\000\010\000\
    \027\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\
    \008\000\008\000\008\000\071\000\073\000\079\000\045\000\010\000\
    \010\000\010\000\010\000\010\000\010\000\010\000\010\000\010\000\
    \010\000\061\000\017\000\038\000\050\000\030\000\104\000\054\000\
    \031\000\110\000\113\000\255\255\019\000\012\000\013\000\016\000\
    \007\000\009\000\014\000\020\000\021\000\023\000\011\000\015\000\
    \007\000\017\000\022\000\255\255\050\000\255\255\255\255\255\255\
    \255\255\007\000\085\000\033\000\036\000\033\000\036\000\255\255\
    \038\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\010\000\076\000\255\255\061\000\255\255\
    \255\255\255\255\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\076\000\038\000\050\000\085\000\
    \017\000\255\255\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
    \017\000\017\000\017\000\017\000\255\255\255\255\255\255\255\255\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\255\255\255\255\255\255\255\255\057\000\082\000\
    \255\255\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\057\000\082\000\097\000\255\255\018\000\
    \000\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\024\000\025\000\030\000\027\000\028\000\
    \031\000\090\000\097\000\255\255\045\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \057\000\082\000\024\000\255\255\255\255\255\255\064\000\102\000\
    \255\255\090\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\064\000\255\255\064\000\102\000\
    \255\255\038\000\050\000\064\000\102\000\255\255\064\000\110\000\
    \113\000\255\255\255\255\064\000\102\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\064\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\090\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\076\000\255\255\255\255\255\255\255\255\255\255\
    \064\000\102\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\057\000\082\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\097\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\090\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\064\000\102\000";
  Lexing.lex_base_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \010\000\000\000\036\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\001\000\000\000\000\000\000\000\000\000\000\000\002\000\
    \062\000\000\000\002\000\002\000\000\000\026\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000";
  Lexing.lex_backtrk_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\020\000\020\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000";
  Lexing.lex_default_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000";
  Lexing.lex_trans_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\001\000\017\000\009\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\009\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \001\000\017\000\009\000\001\000\023\000\000\000\000\000\000\000\
    \000\000\000\000\009\000\000\000\000\000\001\000\000\000\000\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\001\000\000\000\000\000\001\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\001\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\017\000\017\000\001\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check_code = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\007\000\017\000\026\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\008\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \007\000\017\000\026\000\000\000\027\000\255\255\255\255\255\255\
    \255\255\255\255\008\000\255\255\255\255\010\000\255\255\255\255\
    \007\000\007\000\007\000\007\000\007\000\007\000\007\000\007\000\
    \007\000\007\000\008\000\008\000\008\000\008\000\008\000\008\000\
    \008\000\008\000\008\000\008\000\010\000\255\255\255\255\024\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\010\000\010\000\010\000\010\000\
    \010\000\010\000\010\000\010\000\010\000\010\000\024\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\016\000\021\000\023\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_code = 
   "\255\004\255\255\006\255\005\255\255\006\255\255\005\255\006\255\
    \255\007\255\255\000\007\255\008\255\255\000\004\001\005\002\006\
    \003\008\255";
}

let rec main lexbuf =
  lexbuf.Lexing.lex_mem <- Array.make 9 (-1) ;   __ocaml_lex_main_rec lexbuf 0
and __ocaml_lex_main_rec lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
let
# 165 "cil/src/logic/logic_preprocess.mll"
                                                                      m
# 477 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 166 "cil/src/logic/logic_preprocess.mll"
      (
        let blacklisted = List.mem m blacklisted_macros in
        if not blacklisted then
          Buffer.add_string preprocess_buffer (lexeme lexbuf);
        macro blacklisted lexbuf
      )
# 486 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
let
# 172 "cil/src/logic/logic_preprocess.mll"
                                                       line
# 492 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_mem.(1)
and
# 173 "cil/src/logic/logic_preprocess.mll"
                                     file
# 497 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(2) lexbuf.Lexing.lex_mem.(3) in
# 174 "cil/src/logic/logic_preprocess.mll"
    ( (try
        curr_line := (int_of_string line) -1
       with Failure "int_of_string" -> curr_line:= -1);
      if file <> "" then curr_file := file;
      Buffer.add_string output_buffer (lexeme lexbuf);
      make_newline();
      main lexbuf
    )
# 508 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
let
# 182 "cil/src/logic/logic_preprocess.mll"
                        c
# 514 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 3) in
# 182 "cil/src/logic/logic_preprocess.mll"
                           ( (* Skip special doxygen comments. Use of '@'
                                instead of !Clexer.annot_char is intentional *)
        Buffer.add_string beg_of_line "   ";
        Buffer.add_string output_buffer (lexeme lexbuf);
        comment c lexbuf;)
# 522 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
let
# 187 "cil/src/logic/logic_preprocess.mll"
                c
# 528 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 2) in
# 187 "cil/src/logic/logic_preprocess.mll"
                   (
      if c = !Clexer.annot_char then begin
        is_newline:=CHAR;
        has_annot := true;
        Buffer.add_string output_buffer annot_content;
        Buffer.add_string preprocess_buffer annot_beg;
        Buffer.add_char preprocess_buffer '\n';
        add_preprocess_line_info();
        annot lexbuf
      end else begin
        if c = '\n' then make_newline()
        else Buffer.add_string beg_of_line "   ";
        Buffer.add_string output_buffer (lexeme lexbuf);
        comment c lexbuf;
      end)
# 546 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
# 202 "cil/src/logic/logic_preprocess.mll"
                      ( (* See comments for "/*@{" above *)
        Buffer.add_string output_buffer (lexeme lexbuf);
        oneline_comment lexbuf;
      )
# 554 "cil/src/logic/logic_preprocess.ml"

  | 5 ->
let
# 206 "cil/src/logic/logic_preprocess.mll"
                c
# 560 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf (lexbuf.Lexing.lex_start_pos + 2) in
# 206 "cil/src/logic/logic_preprocess.mll"
                   (
      if c = !Clexer.annot_char then begin
        is_newline:=CHAR;
        has_annot:=true;
        Buffer.add_string output_buffer annot_content;
        Buffer.add_string preprocess_buffer annot_beg;
        Buffer.add_char preprocess_buffer '\n';
        add_preprocess_line_info();
        oneline_annot lexbuf
      end
      else if c = '\n' then begin
        make_newline ();
        Buffer.add_string output_buffer (lexeme lexbuf);
        main lexbuf
      end
      else begin
        Buffer.add_string output_buffer (lexeme lexbuf);
        oneline_comment lexbuf;
      end)
# 582 "cil/src/logic/logic_preprocess.ml"

  | 6 ->
# 225 "cil/src/logic/logic_preprocess.mll"
         (
      make_newline (); Buffer.add_char output_buffer '\n'; main lexbuf )
# 588 "cil/src/logic/logic_preprocess.ml"

  | 7 ->
# 227 "cil/src/logic/logic_preprocess.mll"
         ( )
# 593 "cil/src/logic/logic_preprocess.ml"

  | 8 ->
# 228 "cil/src/logic/logic_preprocess.mll"
        (
      Buffer.add_char beg_of_line ' ';
      Buffer.add_char output_buffer '"'; 
      c_string lexbuf )
# 601 "cil/src/logic/logic_preprocess.ml"

  | 9 ->
# 232 "cil/src/logic/logic_preprocess.mll"
        ( 
      Buffer.add_char beg_of_line ' ';
      Buffer.add_char output_buffer '\'';
      c_char lexbuf )
# 609 "cil/src/logic/logic_preprocess.ml"

  | 10 ->
let
# 236 "cil/src/logic/logic_preprocess.mll"
         c
# 615 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 236 "cil/src/logic/logic_preprocess.mll"
           (
      Buffer.add_char beg_of_line ' ';
      Buffer.add_char output_buffer c;
      main lexbuf )
# 622 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_main_rec lexbuf __ocaml_lex_state

and macro blacklisted lexbuf =
    __ocaml_lex_macro_rec blacklisted lexbuf 38
and __ocaml_lex_macro_rec blacklisted lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 241 "cil/src/logic/logic_preprocess.mll"
         (
      if not blacklisted then
        Buffer.add_string preprocess_buffer (lexeme lexbuf);
      make_newline ();
      Buffer.add_char output_buffer '\n';
      macro blacklisted lexbuf
    )
# 640 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 250 "cil/src/logic/logic_preprocess.mll"
       ( macro_comment blacklisted lexbuf )
# 645 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 251 "cil/src/logic/logic_preprocess.mll"
       (
      if not blacklisted then
        Buffer.add_char preprocess_buffer '\n';
      make_newline ();
      Buffer.add_char output_buffer '\n';
      main lexbuf
    )
# 656 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
let
# 258 "cil/src/logic/logic_preprocess.mll"
       c
# 662 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 258 "cil/src/logic/logic_preprocess.mll"
         (
           if not blacklisted then
             Buffer.add_char preprocess_buffer c;
           macro blacklisted lexbuf
         )
# 670 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_macro_rec blacklisted lexbuf __ocaml_lex_state

and macro_comment blacklisted lexbuf =
    __ocaml_lex_macro_comment_rec blacklisted lexbuf 45
and __ocaml_lex_macro_comment_rec blacklisted lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 264 "cil/src/logic/logic_preprocess.mll"
       (
      make_newline ();

      macro_comment blacklisted lexbuf
    )
# 686 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 269 "cil/src/logic/logic_preprocess.mll"
       ( macro blacklisted lexbuf )
# 691 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 270 "cil/src/logic/logic_preprocess.mll"
     ( macro_comment blacklisted lexbuf )
# 696 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_macro_comment_rec blacklisted lexbuf __ocaml_lex_state

and c_string lexbuf =
    __ocaml_lex_c_string_rec lexbuf 50
and __ocaml_lex_c_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 272 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string beg_of_line "  ";
           Buffer.add_string output_buffer (lexeme lexbuf);
           c_string lexbuf )
# 710 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 275 "cil/src/logic/logic_preprocess.mll"
       ( Buffer.add_char beg_of_line ' '; 
         Buffer.add_char output_buffer '"';
         main lexbuf )
# 717 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 278 "cil/src/logic/logic_preprocess.mll"
       ( make_newline ();
         Buffer.add_char output_buffer '\n';
         c_string lexbuf
       )
# 725 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 282 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string beg_of_line "  ";
           Buffer.add_string output_buffer (lexeme lexbuf);
           c_string lexbuf )
# 732 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
let
# 285 "cil/src/logic/logic_preprocess.mll"
       c
# 738 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 285 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_char beg_of_line ' ';
           Buffer.add_char output_buffer c;
           c_string lexbuf )
# 744 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_c_string_rec lexbuf __ocaml_lex_state

and c_char lexbuf =
    __ocaml_lex_c_char_rec lexbuf 57
and __ocaml_lex_c_char_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 290 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string beg_of_line "  ";
           Buffer.add_string output_buffer (lexeme lexbuf);
           c_char lexbuf )
# 758 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 293 "cil/src/logic/logic_preprocess.mll"
      ( Buffer.add_char beg_of_line ' '; 
         Buffer.add_char output_buffer '\'';
         main lexbuf )
# 765 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 296 "cil/src/logic/logic_preprocess.mll"
       ( make_newline ();
         Buffer.add_char output_buffer '\n';
         c_char lexbuf
       )
# 773 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 300 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string beg_of_line "  ";
           Buffer.add_string output_buffer (lexeme lexbuf);
           c_char lexbuf )
# 780 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
let
# 303 "cil/src/logic/logic_preprocess.mll"
       c
# 786 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 303 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_char beg_of_line ' ';
           Buffer.add_char output_buffer c;
           c_char lexbuf )
# 792 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_c_char_rec lexbuf __ocaml_lex_state

and annot lexbuf =
    __ocaml_lex_annot_rec lexbuf 64
and __ocaml_lex_annot_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 308 "cil/src/logic/logic_preprocess.mll"
          (
      if !is_newline = NEWLINE then
        Buffer.add_string preprocess_buffer annot_end_nl
      else begin
        Buffer.add_char preprocess_buffer '\n';
        Buffer.add_string preprocess_buffer annot_end;
      end;
      Buffer.add_char preprocess_buffer '\n';
      main lexbuf )
# 812 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 317 "cil/src/logic/logic_preprocess.mll"
         ( is_newline := NEWLINE;
           incr curr_line;
           Buffer.add_char preprocess_buffer '\n';
           annot lexbuf )
# 820 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 321 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string preprocess_buffer "//";
           annot_comment lexbuf )
# 826 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 323 "cil/src/logic/logic_preprocess.mll"
        (
      if !is_newline = NEWLINE then is_newline:=SPACE;
      Buffer.add_char preprocess_buffer ' ';
      annot lexbuf )
# 834 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
# 327 "cil/src/logic/logic_preprocess.mll"
         (
      if !is_newline = NEWLINE then is_newline:=SPACE;
      Buffer.add_char preprocess_buffer ' ';
      annot lexbuf )
# 842 "cil/src/logic/logic_preprocess.ml"

  | 5 ->
# 335 "cil/src/logic/logic_preprocess.mll"
         ( 
        is_newline := CHAR;
        Buffer.add_string preprocess_buffer backslash;
        annot lexbuf )
# 850 "cil/src/logic/logic_preprocess.ml"

  | 6 ->
# 339 "cil/src/logic/logic_preprocess.mll"
         (
        is_newline := CHAR;
        Buffer.add_char preprocess_buffer '\'';
        char annot lexbuf )
# 858 "cil/src/logic/logic_preprocess.ml"

  | 7 ->
# 343 "cil/src/logic/logic_preprocess.mll"
         (
        is_newline:=CHAR;
        Buffer.add_char preprocess_buffer '"';
        string annot lexbuf )
# 866 "cil/src/logic/logic_preprocess.ml"

  | 8 ->
let
# 347 "cil/src/logic/logic_preprocess.mll"
         c
# 872 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 347 "cil/src/logic/logic_preprocess.mll"
           ( is_newline := CHAR;
             Buffer.add_char preprocess_buffer c;
             annot lexbuf )
# 878 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_annot_rec lexbuf __ocaml_lex_state

and annot_comment lexbuf =
    __ocaml_lex_annot_comment_rec lexbuf 76
and __ocaml_lex_annot_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 352 "cil/src/logic/logic_preprocess.mll"
         ( incr curr_line; is_newline:=NEWLINE;
           Buffer.add_char preprocess_buffer '\n';
           annot lexbuf
         )
# 893 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 356 "cil/src/logic/logic_preprocess.mll"
         (
        Buffer.add_char preprocess_buffer '\n';
        Buffer.add_string preprocess_buffer annot_end;
        Buffer.add_char preprocess_buffer '\n';
        main lexbuf )
# 902 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 361 "cil/src/logic/logic_preprocess.mll"
        ( abort_preprocess "eof in the middle of a comment" )
# 907 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
let
# 362 "cil/src/logic/logic_preprocess.mll"
         c
# 913 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 362 "cil/src/logic/logic_preprocess.mll"
           (
    Buffer.add_char preprocess_buffer c; annot_comment lexbuf )
# 918 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_annot_comment_rec lexbuf __ocaml_lex_state

and char annot lexbuf =
    __ocaml_lex_char_rec annot lexbuf 82
and __ocaml_lex_char_rec annot lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 367 "cil/src/logic/logic_preprocess.mll"
         ( incr curr_line; is_newline:=NEWLINE;
           Buffer.add_char preprocess_buffer '\n';
           char annot lexbuf
         )
# 933 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 371 "cil/src/logic/logic_preprocess.mll"
         ( is_newline:=CHAR;
           Buffer.add_char preprocess_buffer '\'';
           annot lexbuf )
# 940 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 374 "cil/src/logic/logic_preprocess.mll"
          ( is_newline:=CHAR;
            Buffer.add_string preprocess_buffer "\\'";
            char annot lexbuf )
# 947 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 377 "cil/src/logic/logic_preprocess.mll"
           ( is_newline:=CHAR;
            Buffer.add_string preprocess_buffer "\\\\";
            char annot lexbuf )
# 954 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
# 380 "cil/src/logic/logic_preprocess.mll"
        ( abort_preprocess "eof while parsing a char literal" )
# 959 "cil/src/logic/logic_preprocess.ml"

  | 5 ->
let
# 381 "cil/src/logic/logic_preprocess.mll"
         c
# 965 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 381 "cil/src/logic/logic_preprocess.mll"
           ( is_newline:=CHAR;
             Buffer.add_char preprocess_buffer c;
             char annot lexbuf )
# 971 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_char_rec annot lexbuf __ocaml_lex_state

and string annot lexbuf =
    __ocaml_lex_string_rec annot lexbuf 90
and __ocaml_lex_string_rec annot lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 386 "cil/src/logic/logic_preprocess.mll"
         ( incr curr_line; is_newline:=NEWLINE;
           Buffer.add_char preprocess_buffer '\n'; string annot lexbuf
         )
# 985 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 389 "cil/src/logic/logic_preprocess.mll"
        ( is_newline:=CHAR;
          Buffer.add_char preprocess_buffer '"'; annot lexbuf )
# 991 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 391 "cil/src/logic/logic_preprocess.mll"
           ( is_newline:=CHAR;
             Buffer.add_string preprocess_buffer "\\\"";
             string annot lexbuf )
# 998 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 394 "cil/src/logic/logic_preprocess.mll"
        ( abort_preprocess "eof while parsing a string literal" )
# 1003 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
let
# 395 "cil/src/logic/logic_preprocess.mll"
         c
# 1009 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 395 "cil/src/logic/logic_preprocess.mll"
           ( is_newline:=CHAR;
             Buffer.add_char preprocess_buffer c;
             string annot lexbuf )
# 1015 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_string_rec annot lexbuf __ocaml_lex_state

and comment c lexbuf =
    __ocaml_lex_comment_rec c lexbuf 97
and __ocaml_lex_comment_rec c lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 401 "cil/src/logic/logic_preprocess.mll"
        (
      Buffer.add_char beg_of_line ' ';
      Buffer.add_char output_buffer  '/';
      if c = '*' then
        main lexbuf
      else
        comment '/' lexbuf
      )
# 1034 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 409 "cil/src/logic/logic_preprocess.mll"
         ( make_newline (); Buffer.add_char output_buffer '\n';
           comment '\n' lexbuf )
# 1040 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 411 "cil/src/logic/logic_preprocess.mll"
        ( abort_preprocess "eof while parsing C comment" )
# 1045 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
let
# 412 "cil/src/logic/logic_preprocess.mll"
         c
# 1051 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 412 "cil/src/logic/logic_preprocess.mll"
           (
      Buffer.add_char beg_of_line ' ';
      Buffer.add_char output_buffer c;
      comment c lexbuf)
# 1058 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_comment_rec c lexbuf __ocaml_lex_state

and oneline_annot lexbuf =
    __ocaml_lex_oneline_annot_rec lexbuf 102
and __ocaml_lex_oneline_annot_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 418 "cil/src/logic/logic_preprocess.mll"
             (
      incr curr_line;
      Buffer.add_char preprocess_buffer '\n';
      Buffer.add_string preprocess_buffer annot_end_nl;
      Buffer.add_char preprocess_buffer '\n';
      main lexbuf )
# 1075 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
# 424 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string preprocess_buffer backslash;
           oneline_annot lexbuf )
# 1081 "cil/src/logic/logic_preprocess.ml"

  | 2 ->
# 426 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_char preprocess_buffer '\'';
           char oneline_annot lexbuf )
# 1087 "cil/src/logic/logic_preprocess.ml"

  | 3 ->
# 428 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_char preprocess_buffer '"';
           string oneline_annot lexbuf )
# 1093 "cil/src/logic/logic_preprocess.ml"

  | 4 ->
# 430 "cil/src/logic/logic_preprocess.mll"
         ( Buffer.add_string preprocess_buffer "//";
           oneline_annot_comment lexbuf )
# 1099 "cil/src/logic/logic_preprocess.ml"

  | 5 ->
let
# 432 "cil/src/logic/logic_preprocess.mll"
         c
# 1105 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 432 "cil/src/logic/logic_preprocess.mll"
           ( Buffer.add_char preprocess_buffer c;
             oneline_annot lexbuf )
# 1110 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_oneline_annot_rec lexbuf __ocaml_lex_state

and oneline_annot_comment lexbuf =
    __ocaml_lex_oneline_annot_comment_rec lexbuf 110
and __ocaml_lex_oneline_annot_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 436 "cil/src/logic/logic_preprocess.mll"
             (
       incr curr_line;
       Buffer.add_char preprocess_buffer '\n';
       Buffer.add_string preprocess_buffer annot_end_comment;
       Buffer.add_char preprocess_buffer '\n';
       main lexbuf )
# 1127 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
let
# 442 "cil/src/logic/logic_preprocess.mll"
         c
# 1133 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 442 "cil/src/logic/logic_preprocess.mll"
           ( Buffer.add_char preprocess_buffer c;
             oneline_annot_comment lexbuf )
# 1138 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_oneline_annot_comment_rec lexbuf __ocaml_lex_state

and oneline_comment lexbuf =
    __ocaml_lex_oneline_comment_rec lexbuf 113
and __ocaml_lex_oneline_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 448 "cil/src/logic/logic_preprocess.mll"
      ( make_newline();
        Buffer.add_string output_buffer (lexeme lexbuf);
        main lexbuf)
# 1152 "cil/src/logic/logic_preprocess.ml"

  | 1 ->
let
# 451 "cil/src/logic/logic_preprocess.mll"
         c
# 1158 "cil/src/logic/logic_preprocess.ml"
= Lexing.sub_lexeme_char lexbuf lexbuf.Lexing.lex_start_pos in
# 451 "cil/src/logic/logic_preprocess.mll"
           ( Buffer.add_char output_buffer c;
             oneline_comment lexbuf)
# 1163 "cil/src/logic/logic_preprocess.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
      __ocaml_lex_oneline_comment_rec lexbuf __ocaml_lex_state

;;

# 454 "cil/src/logic/logic_preprocess.mll"
 
  let file suffix cpp filename =
    reset ();
    let debug = Kernel.Debug_category.exists (fun x -> x = "parser") in
    let inchan = open_in_bin filename in
    let lex = Lexing.from_channel inchan in
    let ppname =
      Extlib.temp_file_cleanup_at_exit ~debug
        (Filename.basename filename) ".pp"
    in
    let ppfile = open_out ppname in
    main lex;
    preprocess_annots suffix cpp ppfile;
    close_in inchan;
    close_out ppfile;
    ppname

# 1188 "cil/src/logic/logic_preprocess.ml"
